%Generate a constant curvature array
n_elements = 4; %greater than 1
ang_elements = 15; %angular coverage of each element
h_element = 110.5e-3; %height of each element
w_element = 84e-3; %width of each element
h_apature = 0.05115*2; 
w_apature = 0.01305*2;

%work out angle of each element
%0 degrees is center of coverage pattern
angs = abs(-n_elements/2*ang_elements+ang_elements/2:ang_elements:n_elements/2*ang_elements-ang_elements/2);

%plot out vertices of array elements (not centers)
if (rem(n_elements,2) == 0)
  %even number of elements so no centered straight firing
  afp = angs(ceil(n_elements/2):-1:1); %starts from small angle outwards for center outwards point gen
  yoffs = h_element*sind(90-afp);
  zoffs = h_element*cosd(90-afp);
  zpos = [];
  ypos = [];
  zpos(1) = 0;
  ypos(1) = 0;
  for loop = 2:length(zoffs)+1
    zpos(loop) = zpos(loop-1)+zoffs(loop-1);
    ypos(loop) = ypos(loop-1)+yoffs(loop-1);
  endfor
  zpos = -1*[zpos(end:-1:1) zpos(2:end)];
  ypos = [ypos(end:-1:1) -1*ypos(2:end)];
else
  afp = angs(floor(ceil(n_elements/2)-1:-1:1));
  yoffs = h_element*sind(90-afp);
  zoffs = h_element*cosd(90-afp);
  zpos = [];
  ypos = [];
  zpos(1) = 0;
  ypos(1) = h_element/2;
  for loop = 2:length(zoffs)+1
    zpos(loop) = zpos(loop-1)+zoffs(loop-1);
    ypos(loop) = ypos(loop-1)+yoffs(loop-1);
  endfor
  zpos =  -1*[zpos(end:-1:1) zpos];
  ypos = [ypos(end:-1:1) -ypos];
end;

%find centers of elements
zcenter = (zpos(1:end-1)+zpos(2:end))/2;
ycenter = (ypos(1:end-1)+ypos(2:end))/2;
[zc,yc,sR,a] = circfit(zcenter,ycenter); %fit cicle to centers to find effective origin
ca_points = deg2rad(-n_elements*ang_elements/2:1:n_elements*ang_elements/2);
circ_zpoints = zc+sR*cos(ca_points);
circ_ypoints = sR*sin(ca_points);

%output vertices of plane wave elements for simulation
%do this using in plane offsets from centers
hao = h_apature/2;
wao = w_apature/2;
ea_cords = {};
zoff = -hao*sind(angs)  
yoff = hao*cosd(angs)
for loop=1:length(angs)
  if round(loop/(length(angs)+1)) == 0 %upper half
    ea_cords{loop}.uxp = [wao ycenter(loop)+yoff(loop) zcenter(loop)+zoff(loop)];
    ea_cords{loop}.uxn = [-wao ycenter(loop)+yoff(loop) zcenter(loop)+zoff(loop)];
    ea_cords{loop}.lxp = [wao ycenter(loop)-yoff(loop) zcenter(loop)-zoff(loop)];
    ea_cords{loop}.lxn = [-wao ycenter(loop)-yoff(loop) zcenter(loop)-zoff(loop)];
  else %lower half
    ea_cords{loop}.uxp = [wao ycenter(loop)+yoff(loop) zcenter(loop)-zoff(loop)];
    ea_cords{loop}.uxn = [-wao ycenter(loop)+yoff(loop) zcenter(loop)-zoff(loop)];
    ea_cords{loop}.lxp = [wao ycenter(loop)-yoff(loop) zcenter(loop)+zoff(loop)];
    ea_cords{loop}.lxn = [-wao ycenter(loop)-yoff(loop) zcenter(loop)+zoff(loop)];
  endif
  

end;

%Generate 3d plot of arry elements
figure(1);
hold on;
for loop=1:length(angs)
  xcords = [ea_cords{loop}.uxp(1) ea_cords{loop}.uxn(1) ea_cords{loop}.lxn(1) ea_cords{loop}.lxp(1) ea_cords{loop}.uxp(1)];
  ycords = [ea_cords{loop}.uxp(2) ea_cords{loop}.uxn(2) ea_cords{loop}.lxn(2) ea_cords{loop}.lxp(2) ea_cords{loop}.uxp(2)];
  zcords = [ea_cords{loop}.uxp(3) ea_cords{loop}.uxn(3) ea_cords{loop}.lxn(3) ea_cords{loop}.lxp(3) ea_cords{loop}.uxp(3)];
  plot3(xcords, ycords, zcords);
end;
axis equal;
hold off;

%Show array curvature and centers
figure(2);
plot(zpos, ypos)
hold on;
scatter(zcenter, ycenter);
plot(circ_zpoints, circ_ypoints);
hold off;
axis equal;

display('Acoustic origin of array [m]:');
display(zc);

display('Array height [m]:');
display(ypos(1)*2);

display('Array depth [m]');
display(abs(zpos(1)));
